//
//  Complex.cpp
//  audio
//
//  Created by cmtuser on 09/03/15.
//
//

#include <cmath>
#include "Complex.h"

// tyypin rakentaja. Funktio, jolla ei ole palautusarvoa, ja jonka nimi on sama kuin luokan nimi.
// voidaan käyttää Complex-muuttujien luomiseen
Complex::Complex(float r, float i) {
    real = r;
    imaginary = i;
}

// implisiittinen rakentaja; mahdollistaa float-tyyppisen muuttujan muuttamisen Complex-tyyppiseksi
// automaattisesti. Implisiittiset rakentajat ovat yksiparametrisia.
Complex::Complex(float r) {
    real = r;
    imaginary = 0;
}

// oletusrakentaja; lukua ei alusteta.
Complex::Complex( ) { }

// jäsenfunktio. Lasketaan jonkin tietyn Complex-tyyppisen muuttujan itseisarvo.
float Complex::Absolute( ) {
    return sqrtf(real * real + imaginary * imaginary);
}

Complex Complex::Conjugate( ) {
    return Complex(real, -imaginary);
}

// staattinen funktio. Luokan jäsenfunktio, joka ei liity mihinkään tiettyyn Complex-tyyppiseen muuttujaan.
Complex Complex::Polar(float angle, float radius) {
    return Complex(radius * cosf(angle), radius * sinf(angle));
}


// matemaattiset operaattorit Complex-tyypille
Complex operator+(Complex a, Complex b) {
    return Complex(a.real+b.real, a.imaginary+b.imaginary);
}

Complex operator-(Complex a, Complex b) {
    return Complex(a.real-b.real, a.imaginary-b.imaginary);
}

Complex operator*(Complex a, Complex b) {
    return Complex(a.real * b.real - a.imaginary * b.imaginary,
                   a.real * b.imaginary + b.real * a.imaginary);
}

Complex operator/(Complex a, Complex b) {
    Complex num = a * b.Conjugate();
    Complex den = b * b.Conjugate();
    return Complex(num.real / den.real, num.imaginary / den.real);
}

Complex exp(Complex z) {
    // eulerin kaava: exp(z.real) * exp(i z.imaginary)
    return Complex(exp(z.real) * cosf(z.imaginary), exp(z.real) * sinf(z.imaginary));
}
