#pragma once
#include "processor.h"

static const unsigned PlayerSampleRate = 44100;

static void CACallback(void *user, AudioQueueRef AQ, AudioQueueBufferRef Buffer)
{
    Processor *player = (Processor*)user;
    Buffer->mAudioDataByteSize = Buffer->mAudioDataBytesCapacity;
    memset(Buffer->mAudioData,0,Buffer->mAudioDataBytesCapacity);
    float *buf((float*)Buffer->mAudioData);
    for(unsigned i(0);i<Buffer->mAudioDataByteSize/sizeof(float);++i)
    {
        buf[i] = player->Process(0);
    }
    AudioQueueEnqueueBuffer(AQ,Buffer,0,0);
}

static void PlayAudio(const Processor& p, double duration)
{
    Processor& pipeline(const_cast<Processor&>(p));
    AudioQueueRef audioOutputQueue = NULL;
    AudioStreamBasicDescription dataFormat = {PlayerSampleRate,'lpcm',kAudioFormatFlagIsFloat,4,1,4,1,32,0};
    AudioQueueNewOutput(&dataFormat,CACallback,&pipeline,CFRunLoopGetCurrent(),kCFRunLoopCommonModes,0,&audioOutputQueue);
    if (audioOutputQueue)
    {
        for(unsigned i(0);i<3;++i)
        {
            AudioQueueBufferRef buf;
            AudioQueueAllocateBuffer(audioOutputQueue, 2048, &buf); 
            CACallback(&pipeline,audioOutputQueue,buf);
        }
        AudioQueueStart(audioOutputQueue,NULL);
        CFRunLoopRunInMode(kCFRunLoopDefaultMode,duration,false);
        AudioQueueDispose(audioOutputQueue,true);
    }
}

class ProcessorSeq : public Processor{
    Processor &a, &b;
public:
    ProcessorSeq(const Processor &a, const Processor &b):a(const_cast<Processor&>(a)),b(const_cast<Processor&>(b)){}
    float Process(float in) {return b.Process(a.Process(in));}   
    ProcessorSeq operator>>(Processor& b) {return ProcessorSeq(*this,b);}    
};

class ProcessorSum : public Processor{
    Processor &a, &b;
public:
    ProcessorSum(const Processor &a, const Processor &b):a(const_cast<Processor&>(a)),b(const_cast<Processor&>(b)){}
    float Process(float in) {return a.Process(in) + b.Process(in);}
};

static ProcessorSeq operator>>(const Processor& a, const Processor& b) {return ProcessorSeq(a,b);}
static ProcessorSum operator+(const Processor& a, const Processor& b) {return ProcessorSum(a,b);}

class Sine : public Processor{
    double phase;
    double inc;
public:
    Sine(double freq) {phase = 0;inc=M_PI * 2 * freq / (double)PlayerSampleRate;}
    float Process(float) {return sin(phase+=inc);}
};

class Tremolo : public Processor{
    Sine LFO;
public:  
    Tremolo(double freq):LFO(freq) {}
    float Process(float in) {return in * (0.5 + 0.5 * LFO.Process(0));}
};
