#include "soundfileplayer.h"
#include "CAHelper.h"

#include <string>
#include <stdexcept>
#include <iostream>

#define CHECK(expr) {OSStatus code(expr);if (code) throw std::runtime_error(std::string(#expr) + ":" + GetString(code));} 

static std::string GetString(OSStatus code)
{
    if (code>0)
    {
        union{OSStatus c;char str[5];} tmp;    
        tmp.str[4]=0;
        tmp.c=code;    
        tmp.str[4]=0;
        return std::string(tmp.str,tmp.str+5);
    }
    else
    {
        char buf[12];
        sprintf(buf,"%i",code);
        return std::string(buf,buf+12);
    }
}

/*
 struct AudioStreamBasicDescription
 {
 Float64 mSampleRate;
 UInt32  mFormatID;
 UInt32  mFormatFlags;
 UInt32  mBytesPerPacket;
 UInt32  mFramesPerPacket;
 UInt32  mBytesPerFrame;
 UInt32  mChannelsPerFrame;
 UInt32  mBitsPerChannel;
 UInt32  mReserved;
 };
 */

SoundfileRecorder::SoundfileRecorder(const char *path) {
    CFStringRef filePath = CFStringCreateWithCString(NULL,path,kCFStringEncodingUTF8);
    CFURLRef fileUrl = CFURLCreateWithFileSystemPath(NULL,filePath,kCFURLPOSIXPathStyle,false);
    try {
        AudioStreamBasicDescription dataFormat = {PlayerSampleRate,'lpcm',kAudioFormatFlagIsSignedInteger | kAudioFormatFlagsNativeEndian | kAudioFormatFlagIsPacked,2,1,2,1,16,0};
        AudioStreamBasicDescription clientFormat = {PlayerSampleRate,'lpcm',kAudioFormatFlagIsFloat,4,1,4,1,32,0};
        CHECK(ExtAudioFileCreateWithURL(fileUrl, 'WAVE', &dataFormat, nullptr, 0, &audioFile));
        CHECK(ExtAudioFileSetProperty(audioFile,kExtAudioFileProperty_ClientDataFormat,sizeof(clientFormat),&clientFormat));
    }  catch (std::runtime_error& e) {
        if (audioFile) ExtAudioFileDispose(audioFile);
        if (filePath) CFRelease(filePath);
        if (fileUrl) CFRelease(fileUrl);
        std::cout << e.what() << std::endl;
        throw;
    }
    buffer.resize(0x1000);
    counter = 0;
}

float SoundfileRecorder::Process(float input) {
    buffer[counter++] = input;
    if ( counter == buffer.size()) {
        AudioBufferList list;
        AudioBuffer buf;
        buf.mNumberChannels = 1;
        buf.mDataByteSize = (unsigned)buffer.size() * sizeof(float);
        buf.mData = buffer.data();
        list.mNumberBuffers = 1;
        list.mBuffers[0] = buf;
        ExtAudioFileWrite(audioFile, (UInt32)buffer.size(), &list);
        counter = 0;
    }
    return input;
}

SoundfileRecorder::~SoundfileRecorder() {
    if (audioFile) ExtAudioFileDispose(audioFile);
}

SoundfilePlayer::SoundfilePlayer(const char *path)
{
    counter = 0;    
    ExtAudioFileRef audioFile = 0;
    CFStringRef filePath = CFStringCreateWithCString(NULL,path,kCFStringEncodingUTF8);
    CFURLRef fileUrl = CFURLCreateWithFileSystemPath(NULL,filePath,kCFURLPOSIXPathStyle,false);
    
    try{
        CHECK(ExtAudioFileOpenURL(fileUrl,&audioFile));
        AudioStreamBasicDescription dataFormat = {PlayerSampleRate,'lpcm',kAudioFormatFlagIsFloat,4,1,4,1,32,0};
        CHECK(ExtAudioFileSetProperty(audioFile,kExtAudioFileProperty_ClientDataFormat,sizeof(dataFormat),&dataFormat));
        
        SInt64 fileSize;
        UInt32 fileSizeSize(sizeof(SInt64));
        CHECK(ExtAudioFileGetProperty(audioFile,kExtAudioFileProperty_FileLengthFrames,&fileSizeSize,&fileSize));
        
        sound.resize(fileSize);
        
        AudioBufferList list;
        AudioBuffer buf;
        buf.mNumberChannels = 1;
        buf.mDataByteSize = (unsigned)fileSize * sizeof(float);       
        buf.mData = sound.data();
        list.mNumberBuffers = 1;
        list.mBuffers[0] = buf;
        UInt32 frames = (UInt32)fileSize;
        CHECK(ExtAudioFileRead(audioFile,&frames,&list));     
    }
    catch (std::runtime_error& e) {
        if (audioFile) ExtAudioFileDispose(audioFile);
        if (filePath) CFRelease(filePath); 
        if (fileUrl) CFRelease(fileUrl);
        std::cout << e.what() << std::endl;
        throw;
    }
    if (audioFile) ExtAudioFileDispose(audioFile);
}

float SoundfilePlayer::Process(float in)
{
    if (counter >= sound.size()) counter = 0;
    return sound[counter++];
}